<?php
/*===================================*/
/*= Mathieu Chartier
/*= Version 1.0
/*= https://www.mathieu-chartier.com
/*===================================*/

/*========================================================*/
/*= Fonction simple : print_recursive() avec 3 arguments
/*= Elle affiche la liste des clés/valeurs par niveau
/*= 1. Tableau/Objet à parcourir
/*= 2. Mise en gras des clés affichées (True/False)
/*= 3. Niveau de profondeur (Ne pas modifier en théorie)
/*========================================================*/
function print_recursive($tab = array(), $strong = false, $niveau = 0) {
	// Supprime les clés private et protected s'il s'agit d'un objet
	if(is_object($tab)) {
		$tab = get_object_vars($tab);
	}

	// Ajoute les sauts de ligne pour chaque niveau de profondeur (sauf le premier !)
	if($niveau != 0) {
		echo "<br/>";
	}

	foreach($tab as $key => $value) {
		// Affichage des clés avec tabulation par niveau de profondeur
		if($strong === true) {
				echo str_repeat("&nbsp;", $niveau * 4)."<strong>".$key."</strong> => ";
			} else {
				echo str_repeat("&nbsp;", $niveau * 4).$key." => ";
			}

		// Vérifie si la valeur correspond elle-même à un tableau ou un objet
		if((is_array($value) || is_object($value))) {
			// Appel récursif de la fonction pour lire un autre niveau de profondeur
			print_recursive($value, $strong, $niveau + 1);
			continue;
		}

		// Affichage des valeurs (quand ce ne sont pas des tableaux/objets)
		echo $value."<br/>";
	}
}

/*===============================================================*/
/*= Fonction double effet : print_recursive_tab() -> 4 arguments
/*= Elle permet de retourner un tableau par niveau
/*= 1. Tableau/Objet à parcourir
/*= 2. Affichage des clés/valeurs par niveau (True/False)
/*= 3. Mise en gras des clés affichées (True/False)
/*= 4. Niveau de profondeur (Ne pas modifier en théorie)
/*===============================================================*/
function print_recursive_tab($tab = array(), $echo = false, $strong = false, $niveau = 0) {
	// Crée un tableau statique pour enregistrer chaque niveau de profondeur
	static $new_tab = array();

	// Ajoute les sauts de ligne pour chaque niveau de profondeur (sauf le premier !)
	if($echo === true && $niveau != 0) {
		echo "<br/>";
	}

	foreach($tab as $key => $value) {
		if($echo == true) {
			// Affichage des clés avec tabulation par niveau de profondeur
			if($strong === true) {
				echo str_repeat("&nbsp;", $niveau * 4)."<strong>".$key."</strong> => ";
			} else {
				echo str_repeat("&nbsp;", $niveau * 4).$key." => ";
			}
		}

		// Enregistre les données niveau de profondeur par niveau de profondeur
		if(!is_object($value)) {
			$new_tab[$niveau][$key] = $value;
		} else {
			$new_tab[$niveau][$key] = get_object_vars($value);
		}

		if((is_array($value) || is_object($value))) {
			// Appel récursif de la fonction pour lire un autre niveau de profondeur
			print_recursive_tab($value, $echo, $strong, $niveau + 1);
			continue;
		}
		
		if($echo == true) {
			// Affichage des valeurs (quand ce ne sont pas des tableaux/objets)
			echo $value."<br/>";
		}
	}

	// Retourne le tableau pour chaque niveau de profondeur
	return $new_tab;
}

/*================================================================*/
/*= Fonction complexe : print_recursive_callback() -> 4 arguments
/*= Elle retourne un affichage personnalisable par callback
/*= 1. Tableau/Objet à parcourir
/*= 2. Fonction de retour (callback) pour personnaliser le rendu
/*= 3. Utiliser ou non les clés dans le callback (true/false)
/*= 4. Niveau de profondeur (Ne pas modifier en théorie)
/*= Le callback prend 3 paramètres
/*= $withKey => function cbk($cle, $valeur, $ niveau) { ... }
/*================================================================*/
function print_recursive_callback($tab = array(), $callback, $withKey = true, $niveau = 0) {
	// Supprime les clés private et protected s'il s'agit d'un objet
	if(is_object($tab)) {
		$tab = get_object_vars($tab);
	}

	foreach($tab as $key => $value) {
		if((is_array($value) || is_object($value))) {
			// Appel récursif de la fonction pour lire un autre niveau de profondeur
			if($withKey === true) {
				call_user_func($callback, $key, "", $niveau);
			}
			print_recursive_callback($value, $callback, $withKey, $niveau + 1);
			continue;
		}

		// Appel à la fonction de rappel avec quatre paramètres oblibatoires !!!
		if($withKey === true) {
			call_user_func($callback, $key, $value, $niveau);
		} else {
			call_user_func($callback, $key, $value, $niveau);
		}
	}
}

/*====================================================*/
/*= Test => Tableau multidimension avec objet intégré
/*====================================================*/
class Test {
	public $value = "Valeur 6 publique";
	protected $value2 = "Valeur 6 protégée";
	private $value3 = "Valeur 6 privée";

	public function __construct($valeur = "Valeur 6 publique") {
		$this->value = $valeur;
	}
}

$tab = array(
	"cle1" => array(
		"sous-cle-1A" => "sous-valeur1A",
		"sous-cle-1B" => "sous-valeur1B",
	),
	"cle2" => "valeur2",
	"cle3" => array(
		"sous-cle1" => "sous-valeur1",
		"sous-cle2" => array(
			"sous-sous-cle1" => "sous-sous-valeur1",
			"sous-sous-cle2" => new Test("sous-sous-valeur2"),
		),
		"sous-cle3" => "sous-valeur3",
	),
	"cle4" => array(
		array(
			"sous-cle1B" => "sous-valeur1B",
			"sous-cle2B" => "sous-valeur2B",
		)
	),
	"cle5" => "valeur5",
	"cle6" => new Test()
);

/*====================================================*/
/*= Test => Tableau multidimension avec objet intégré
/*====================================================*/
// Affichage simple
print_recursive($tab);

// Affichage avec tableau en retour par niveau de profondeur
$result = print_recursive_tab($tab);
// var_dump($result);

// Affichage avec fonction de rappel (callback)
function callback($key, $value, $niveau) {
	// Même affichage qu'avec print_recursive()
	// echo str_repeat("&nbsp;", $niveau * 4).$key." => ".$value."<br/>";

	// Variante d'affichage
	if($niveau == 1) {
		echo "<tr style='background:#bbb'>";
		echo "<td>".$key."</td>";
		echo "<td>".$value."</td>";
		echo "</tr>";
	} else if($niveau == 2) {
		echo "<tr style='background:#ddd'>";
		echo "<td>".$key."</td>";
		echo "<td>".$value."</td>";
		echo "</tr>";
	} else if($niveau == 3) {
		echo "<tr style='background:#fff'>";
		echo "<td>".$key."</td>";
		echo "<td>".$value."</td>";
		echo "</tr>";
	} else {
		echo "<tr>";
		echo "<td>".$key."</td>";
		echo "<td>".$value."</td>";
		echo "</tr>";
	}
}
echo "<table style='background:#999'>";
print_recursive_callback($tab, "callback");
echo "</table>";
?>